@@ -1,6 +1,6 @@
 /*
  * Default accounts check
- * $Id: accounts.c,v 1.6 1999/11/07 21:04:17 renaud Exp $
+ * $Id: accounts.c,v 1.12 2000/05/04 20:09:30 renaud Exp $
  *
  * By Renaud Deraison <deraison@cvs.nessus.org>
  *
@@ -13,10 +13,10 @@
 #include <includes.h>
 #include "structs.h"
 #include "read_accounts.h"
-#include "accounts.h"
 #include "network.h"
 
 
+
 #define EN_NAME "Default accounts"
 #define FR_NAME "Comptes par dfaut"
 
@@ -42,7 +42,7 @@ on fonctionns ou pas"
 
 
 #define FILE_LOCATION "Accounts location : "
-#define FILE_DFL_LOC DIR_PREFIX"/share/nessus/accounts.txt"
+#define FILE_DFL_LOC CONFIGDIR"/nessus/accounts.txt"
 
 #define NUM_CONNECTIONS "Simultaneous connections : "
 #define DFL_CONNECTIONS "10"
@@ -54,20 +54,28 @@ on fonctionns ou pas"
 int plugin_init(desc)
  struct arglist * desc;
 {
- plug_set_name(desc, FR_NAME, "francais");
- plug_set_name(desc, EN_NAME, NULL);
+ plug_set_id(desc, 10328);
+ plug_set_cve_id(desc, "CAN-1999-0502");
+ plug_set_name(desc, "Comptes par dfaut", "francais");
+ plug_set_name(desc, "Default accounts", NULL);
  
  
- plug_set_description(desc, FR_DESC, "francais");
- plug_set_description(desc, EN_DESC, NULL);
+ plug_set_description(desc, "\
+Ce plugin test diffrentes combinaisons de nom\n\
+d'utilsateur/mot de passe et indique si les essais\n\
+on fonctionns ou pas", "francais");
+ plug_set_description(desc, "\
+This plugin checks several the login/password\n\
+combinations, and reports whether the attempts\n\
+were successful or not", NULL);
  
- plug_set_summary(desc, FR_SUMM, "francais");
- plug_set_summary(desc, EN_SUMM, NULL);
+ plug_set_summary(desc, "Fait un telnet sur l'hote distant et essaye des login/passwd", "francais");
+ plug_set_summary(desc, "Telnet to the remote host and try login/passwords", NULL);
  
- plug_set_copyright(desc, COPY, NULL);
+ plug_set_copyright(desc, "This plugin was written by Renaud Deraison", NULL);
  
- plug_set_family(desc, FR_FAMILY, "francais");
- plug_set_family(desc, EN_FAMILY, NULL);
+ plug_set_family(desc, "Divers", "francais");
+ plug_set_family(desc, "Misc.", NULL);
  
  
  plug_set_category(desc, ACT_GATHER_INFO);
@@ -75,7 +83,9 @@ int plugin_init(desc)
  /*
   * Preferences
   */
+#if 0
   add_plugin_preference(desc, FILE_LOCATION, PREF_ENTRY, FILE_DFL_LOC);
+#endif  
   add_plugin_preference(desc, NUM_CONNECTIONS, PREF_ENTRY, DFL_CONNECTIONS);
 
   plug_require_port(desc, "Services/telnet");
@@ -88,6 +98,7 @@ plugin_run(data)
  struct arglist * data;
 {
  int port = (int)plug_get_key(data, "Services/telnet");
+ int error = 0;
  int s;
  
  if(port)port = atoi((char*)port);
@@ -97,7 +108,11 @@ plugin_run(data)
  if(s<0)return(0);
  else 
  {
+#if 0
  char * filename = get_plugin_preference(data, FILE_LOCATION);
+#else
+ char * filename = strdup(FILE_DFL_LOC);
+#endif 
  char * asc_num  = get_plugin_preference(data, NUM_CONNECTIONS);
  int num = atoi(asc_num);
  int * sockets;
@@ -105,32 +120,36 @@ plugin_run(data)
  struct accounts ** accounts = read_accounts(filename, &num_accs);
  char ** ret;
  int i;
+ int retry  = 0;
  
  
  shutdown(s, 2);
  close(s);
+ if(!accounts)exit(0);
  qsort(accounts, num_accs, sizeof(struct accounts*),accs_compar);
  while(1)
  {
   int limit;
-  
   for(limit=0;!(accounts[limit]->tested) && (limit < num);limit++);
   if(!limit)break;
   
-  
+ 
  sockets = init_telnet_connections(data, port, limit, 1);
  ret = read_all(sockets);
  for(i=0;i<limit;i++){
- 	if(!strstr(ret[i], "ogin:"))
+	if(ret[i] && (!strstr(ret[i], "ogin:")))
 	{
+        if(!strstr(ret[i], "ogin:"))
+         {
 	 shutdown(sockets[i], 2);
 	 close(sockets[i]);
 	 sockets[i]=-1;
+         }
+        free(ret[i]);
 	}
-	free(ret[i]);
      }
+
  free(ret);
- 
  for(i=0;i<limit;i++)
  {
   int flag = 0;
@@ -143,16 +162,20 @@ plugin_run(data)
    free(data);
   }
   /* no socket open. Is that a _real_ telnet ? */
-  if(!flag)return;
+  if(!flag){
+	retry++;
+	if(retry > 5)return;
+	continue;
+	}
  }
  
 
  
  ret = read_all(sockets);
  
- 
  for(i=0;i<limit;i++)
  {
+#if 0   
   if(!(ret[i] && strstr(ret[i], "word:")))
   {
    if(ret[i])
@@ -166,7 +189,8 @@ plugin_run(data)
    close(sockets[i]);
    sockets[i]=-1;
   }
-  free(ret[i]);
+#endif
+  if(ret[i])free(ret[i]);
  }
  free(ret);
  
@@ -182,6 +206,29 @@ plugin_run(data)
   }
  }
  
+ ret = read_all(sockets);
+ for(i=0;i<limit;i++)
+ {
+  if(ret[i]){
+  	free(ret[i]);
+	ret[i] = NULL;
+	}
+ }
+ free(ret);
+ 
+ for(i=0;i<limit;i++)
+ {
+  if(sockets[i]!=-1)
+  {
+   char * data = emalloc(5);
+   sprintf(data, "id\r\n", accounts[i]->pass);
+   accounts[i]->tested = 1;
+   send(sockets[i], data, strlen(data), 0);
+   free(data);
+  }
+ }
+ 
+ 
  ret = read_all(sockets);
  for(i=0;i<limit;i++)
  {
@@ -189,11 +236,15 @@ plugin_run(data)
   {
    int j,k = strlen(ret[i]);
    for(j=0;j<k;j++)ret[i][j]=tolower(ret[i][j]);
-   if((strlen(ret[i])>5)&&((!strstr(ret[i], "denied"))&&
-      (!strstr(ret[i], "incorrect"))&&
-      (!strstr(ret[i], "bad "))&&
-      (!strstr(ret[i], "ogin:")))||strstr(ret[i], "last login"))
+   if((strstr(ret[i], "uid="))  ||
+      (strstr(ret[i], "command not found")))
       {
+       char * report;
+       report = emalloc(1024);
+       sprintf(report, "The account %s/%s seems to be valid. Change or disable it\n",
+    			accounts[i]->login, accounts[i]->pass);
+        post_hole(data, port, report);
+       free(report);
        accounts[i]->working = 1;
       }
   free(ret[i]);
@@ -210,21 +261,9 @@ plugin_run(data)
 
 
  qsort(accounts, num_accs, sizeof(struct accounts*),accs_compar);
- if(accounts[0]->tested)break;
+ if(accounts[0]->tested) break;
  }
 
-for(i=0;i<num_accs;i++)
-{
- if(accounts[i]->working)
- {
-  char * report;
-  report = emalloc(1024);
-  sprintf(report, "The account %s/%s seems to be valid. Change or disable it\n",
-  			accounts[i]->login, accounts[i]->pass);
-  post_hole(data, port, report);
-  free(report);
- }
-}
 }
 }
  
