@@ -41,6 +41,8 @@ Facteur de risque : Aucun"
 int plugin_init(desc)
  struct arglist * desc;
 { 
+ plug_set_id(desc, 10330);
+ 
  plug_set_name(desc, FR_NAME, "francais");
  plug_set_name(desc, EN_NAME, NULL);
  
@@ -75,6 +77,25 @@ void mark_nntp_server(desc, port, buffer)
 }
 
 
+void mark_swat_server(desc, port, buffer)
+struct arglist * desc;
+ int port;
+ char * buffer;
+{
+ plug_set_key(desc, "Services/swat", ARG_INT, (void*)port);
+}
+
+void mark_vqserver(desc, port, buffer)
+struct arglist * desc;
+ int port;
+ char * buffer;
+{
+ plug_set_key(desc, "Services/vqServer-admin", ARG_INT, (void*)port);
+}
+
+
+
+
 void mark_http_server(desc, port, buffer)
  struct arglist * desc;
  int port;
@@ -197,6 +218,53 @@ mark_imap_server(desc, port, buffer)
  if(port != 143)post_info(desc, port, "an IMAP server is running on this port");
 }
 
+void
+mark_auth_server(desc, port, buffer)
+ struct arglist * desc;
+ int port;
+ char * buffer;
+{
+ plug_set_key(desc, "Services/auth", ARG_INT, (void *)port);
+ if(port != 113)post_info(desc, port, "an identd server is running on this port");
+}
+
+void
+mark_wild_shell(desc, port, buffer)
+ struct arglist * desc;
+ int port;
+ char * buffer;
+{
+
+ plug_set_key(desc, "Services/wild_shell", ARG_INT, (void*)port);
+ 
+ post_hole(desc, port, "a shell seems to be running on this port ! (this is a possible backdoor)");
+}
+
+void
+ mark_telnet_server(desc, port, buffer)
+ struct arglist * desc;
+ int port;
+ char * buffer;
+{
+
+ plug_set_key(desc, "Services/telnet", ARG_INT, (void*)port);
+ 
+ if(port != 23)
+    post_info(desc, port, "a telnet server seems to be running on this port");
+}
+
+void
+ mark_netbus_server(desc, port, buffer)
+ struct arglist * desc;
+ int port;
+ char * buffer;
+{
+
+ plug_set_key(desc, "Services/netbus", ARG_INT, (void*)port);
+ post_hole(desc, port, "NetBus is running on this port");
+}
+
+
 void
 mark_linuxconf(desc, port, buffer)
  struct arglist * desc;
@@ -213,10 +281,18 @@ int plugin_run(desc)
 {
  struct arglist * h = arg_get_value(desc, "HOSTNAME");
  char * head = "Ports/tcp/";
+ u_short unknown[65535];
+ int num_unknown = 0;
+ 
  int len_head = strlen(head);
+ 
+ bzero(unknown, sizeof(unknown));
+
  if(!h)return(1);
+ 
  h = arg_get_value(desc, "key");
  if(!h)return(1);
+
  /*
   * Search for a web server and stop when one is found
   */
@@ -233,40 +309,90 @@ int plugin_run(desc)
   soc = open_sock_tcp(desc, port);
   if(soc >= 0)
   {
-   sprintf(buffer, "HEAD / HTTP/1.0\n\n");
+   sprintf(buffer, "GET / HTTP/1.0\r\n\r\n");
    send(soc, buffer, strlen(buffer), 0);
    FD_ZERO(&fd);
    FD_SET(soc, &fd);
+    bzero(buffer, 2048);
    if(select(soc+1, &fd, NULL, NULL, &tv)>0)
    {
     int i;
-    int len;
-    bzero(buffer, 2000);
-    recv(soc, buffer, 2048, 0);
-    len = strlen(buffer);
-    for(i=0;i<len;i++)buffer[i] = tolower(buffer[i]);
-    if(strstr(buffer, "smtp"))
+    int len = 0;
+    int ok = 1;
+      char * line;
+    while(ok)
+    {
+    int n;
+  
+    
+    n = recv(soc, buffer+len, 2047-len, 0);
+    if(n > 0)len+=n;
+    bzero(&tv, sizeof(tv));
+    tv.tv_sec = 1;
+    FD_ZERO(&fd);
+    FD_SET(soc, &fd);
+    
+    if((len >= 2047)||(n == 0))ok = 0;
+    else ok = (select(soc+1, &fd, NULL, NULL, &tv)>0);
+    }
+    
+    
+ 
+   for(i=0;i<len;i++)buffer[i] = tolower(buffer[i]);
+   line = strdup(buffer);
+   if(strchr(line, '\n')){
+   	char * t = strchr(line, '\n');
+	t[0]='\0';
+	}
+    
+    /*
+     * Many services run on the top of an HTTP protocol,
+     * so the HTTP test is not an 'ELSE ... IF'
+     */
+    if(!strncmp(line, "http/1.", 7) && 
+       !strstr(buffer, "server: linuxconf/"))
+      mark_http_server(desc, port, buffer);
+   
+   
+    if(((u_char)buffer[0]==255) && ((u_char)buffer[1]==253))
+      mark_telnet_server(desc, port, line);
+    else  if(strstr(line, "smtp"))
     	mark_smtp_server(desc, port, buffer);	
     else if(strstr(buffer, "rmserver")||strstr(buffer, "realserver"))
-        mark_rmserver(desc, port, buffer);
-    else if(strstr(buffer, "http")&&strstr(buffer, "200"))
-    	mark_http_server(desc, port, buffer);   
-    else if(strstr(buffer, "ftp"))
-    	mark_ftp_server(desc, port, buffer);	
-    else if(strstr(buffer, "ssh"))
-    	mark_ssh_server(desc, port, buffer);
-    else if(strstr(buffer, "+ok"))
-    	mark_pop_server(desc, port, buffer);
-    else if(strstr(buffer, "* ok imap4"))
-    	mark_imap_server(desc, port, buffer);
-      else if(strstr(buffer, "inn ")||strstr(buffer, "posting OK"))
-    	mark_nntp_server(desc, port, buffer);	
+        mark_rmserver(desc, port, line);
+    else if(strstr(line, "ftp") && !strncmp(line, "220", 3))
+    	mark_ftp_server(desc, port, line);	
+    else if(strstr(line, "ssh-"))
+    	mark_ssh_server(desc, port, line);
+    else if(strstr(line, "+ok"))
+    	mark_pop_server(desc, port, line);
+    else if(strstr(line, "* ok imap4"))
+    	mark_imap_server(desc, port, line);
+      else if(strstr(line, "inn ")||strstr(line, "posting OK"))
+    	mark_nntp_server(desc, port, line);	
     else if(strstr(buffer, "networking/linuxconf"))
-        mark_linuxconf(desc, port, buffer);	
-    else if(strstr(buffer, "http")&&strstr(buffer, "cache")&&
+        mark_linuxconf(desc, port, line);
+   else if(strstr(buffer, "www-authenticate: basic realm=\"swat\""))
+   	mark_swat_server(desc, port, line);
+   else if(strstr(buffer, "vqServer") &&
+           strstr(buffer, "www-authenticate: basic realm=/"))
+	 mark_vqserver(desc,port, line);
+   else if(strstr(buffer, "get: command not found"))
+   	mark_wild_shell(desc, port, line);	
+   else if(strstr(buffer, "netbus"))
+   	mark_netbus_server(desc, port, line);	
+   else if(strstr(line, "0 , 0 : error : unknown-error"))
+   	mark_auth_server(desc, port, line);
+    else if(strstr(line, "http")&&strstr(buffer, "cache")&&
     	    strstr(buffer, "error"))
-	mark_http_proxy(desc, port, buffer);	
+	mark_http_proxy(desc, port, line);	
+    else unknown[num_unknown++] = port;
+    
+    
+   free(line); 
    }
+   
+
    shutdown(soc, 2);
    close(soc);
    efree(&buffer);
@@ -274,5 +400,16 @@ int plugin_run(desc)
   }
   if(h)h = h->next;
   }
+    
+    
+  if(num_unknown)
+   {
+    int i = 0;
+    while(unknown[i])
+      {
+      plug_set_key(desc, "Services/unknown", ARG_INT,(void*)((int)unknown[i]));
+      i++;
+      }
+   } 
  return(0);
 }
