@@ -9,7 +9,6 @@
  */
  
 #include <includes.h>
-#include "queso.h"
 
 #define NAME "QueSO"
 #define EN_DESCRIPTION "\
@@ -45,7 +44,7 @@ Facteur de risque : faible"
 
 
 #define FILE_LOCATION "queso.conf location : "
-#define FILE_DFL_LOC DIR_PREFIX"/share/nessus/queso.conf"
+#define FILE_DFL_LOC CONFDIR"/nessus/queso.conf"
 extern char * queso_main(struct arglist *, struct in_addr , int, char *, char *);
 
 
@@ -53,6 +52,9 @@ int
 plugin_init(desc)
  struct arglist * desc;
 {
+
+ plug_set_id(desc, 10337);
+ plug_set_cve_id(desc, "CAN-1999-0454");
  plug_set_name(desc, NAME, NULL);
  plug_set_description(desc, FR_DESCRIPTION, "francais");
  plug_set_description(desc, EN_DESCRIPTION, NULL);
@@ -67,7 +69,9 @@ plugin_init(desc)
  
  
  plug_set_category(desc, ACT_GATHER_INFO);
+#if 0 
  add_plugin_preference(desc, FILE_LOCATION, PREF_ENTRY, FILE_DFL_LOC);
+#endif 
  plug_exclude_key(desc, "Host/OS");
  return(0);
 }
@@ -78,7 +82,7 @@ plugin_run(desc)
 {
  struct in_addr * host = plug_get_host_ip(desc);
  int port = plug_get_host_open_port(desc);
- char * filename = get_plugin_preference(desc, FILE_LOCATION);
+ char * filename = strdup(FILE_DFL_LOC);
  char * report;
  struct in_addr src;
  char * name = emalloc(255);
